import * as twgl from '../../public/vendor/twgl/twgl-full.module.js'

const identity = twgl.m4.identity()
const id = (size, m) => {
    const len = m.length
    for (let i = 0; i < len; i += size) {
        m.set(identity, i)
    }
    return m
}
export const Range = size => ({
    min: new Float32Array(size),
    max: new Float32Array(size)
})

export const UVector2 = size => []
export const UVector3 = size => []

export const Vector2 = size => []
export const Vector3 = size => []

export const Matrix3x3 = size => []
export const Matrix4x4 = size => []

export class ComponentProxy {
    id
    _component

    /**
    * @param {number} id
    * @param {import('../ecs.js').Component} component
    */
    constructor(id, component) {
        this.id = id
        this._component = component
    }

    valueOf() {
        return this._component[this.id]
    }
}

export class ArrayComponentProxy extends ComponentProxy {
    _length

    /**
    * @param {number} id
    * @param {import('../ecs.js').Component} component
    * @param {number} length 
    */
    constructor(id, component, length) {
        super(id, component)
        this._length = length
    }

    valueOf() {
        return this._component.subarray(this.id, this.id + this._length)
    }
}

export class Vector2Proxy extends ArrayComponentProxy {
    /* 
    * @param {number} id
    * @param {import('../ecs.js').Component} [component=Vector2]
    * @param {number} [length=2] 
    */
    constructor(id, component = Vector2, length = 2) {
        super(id, component, length)
    }

    get x() {
        return this._component[this.id]
    }

    get y() {
        return this._component[this.id + 1]
    }
}

export class Vector3Proxy extends Vector2Proxy {
    /* 
    * @param {number} id
    * @param {import('../ecs.js').Component} [component=Vector3]
    * @param {number} [length=3] 
    */
    constructor(id, component = Vector3, length = 3) {
        super(id, component, length)
    }

    get z() {
        return this._component[this.id + 2]
    }
}

export class Matrix3x3Proxy extends ArrayComponentProxy {
    constructor(id, component = Matrix3x3) {
        super(id, component, 9)
    }
}

export class Matrix4x4Proxy extends ArrayComponentProxy {
    constructor(id, component = Matrix4x4) {
        super(id, component, 16)
    }
}
