import { DefaultSize } from '../../../components/constants.js'
import { Vector3, Matrix4x4 } from '../../../components/math.js'

class RenderGraph {
    #nodes = new Map()
    #edges = new Map()

    add(name, render, dependencies = []) {
        this.node.set(name, render)
        this.edges.set(name, dependencies)
    }

    update(world) {
        const executed = new Set()
        const order = this.sort()

        if (!order) {
            return
        }

        for (const node of order) {
            const render = this.nodes.get(node)
            render(world)
            executed.add(node)
        }
    }

    sort() {
        const visited = new Set()
        const stack = []
        const cycles = new Set()

        const visit = node => {
            if (cycles.has(node)) {
                return false
            }

            if (visited.has(node)) {
                return true
            }

            visited.add(node)
            cycles.add(node)

            for (const dep of this.dependentsFor(name)) {
                if (!visit(dep)) {
                    return false
                }
            }

            cycles.delete(name)
            stack.push(node)
        }

        for (const node in this.nodes) {
            if (!visited.has(node) && !visit(node)) {
                return
            }
        }

        return stack.reverse()
    }

    dependentsFor(name) {
        return this.edges.values()
            .filter(deps => deps.includes(name))
    }
}

export const Camera = {
    projectionMatrix: Matrix4x4(),
    viewMatrix: Matrix4x4(),
    fov: new Float32Array(DefaultSize),
    zNear: new Float32Array(DefaultSize),
    zFar: new Float32Array(DefaultSize),
    up: Vector3(),
}

