import { World } from '../../../core/engine.js'
import { query } from '../../../ecs.js'
import { Transform } from '../../transform/components/index.js'
import { Camera } from '../components/index.js'
import { WebGLContext } from '../../renderer/webgl-context.js'
import * as twgl from '/public/vendor/twgl/twgl-full.module.js'

/** @param {World} world */
export const updateCamera = world => {
    /** @type {WebGLContext} */
    const glContext = world.getResource(WebGLContext)

    /** @type {HTMLCanvasElement} */
    // @ts-ignore
    const canvas = glContext.context.canvas

    /** @param {World} world */
    return world => {
        const cameras = query(world, [Camera, Transform])
        const len = cameras.length

        for (let i = 0; i < len; i++) {
            const entity = cameras[i]

            const projection = Camera.projectionMatrix[entity]
            const transform = Transform[entity]
            const view = Camera.viewMatrix[entity]

            const fov = Camera.fov[entity]
            const aspect = canvas.clientWidth / canvas.clientHeight
            const zNear = Camera.zNear[entity]
            const zFar = Camera.zFar[entity]

            twgl.m4.perspective(fov, aspect, zNear, zFar, projection)
            twgl.m4.inverse(transform, view)
        }
    }
}

